/* 
baby box  27.10.2018
1. copy MP3 files to SD card, adjust number of files below (SONG_COUNT)
2. touch box with TAG, rotate box left/right to change song for TAG

Arduino Micro  -OR-    Arduino Nano-- module
5v (RAW)               5V          -- MPU6050 VCC
GND                    GND         -- MPU6050 GND
D3                     A5          -- MPU6050 SCL   
D2                     A4          -- MPU6050 SDA            
D5                     D5          -- powerOffSwitch

D10                    D10         -- MFRC522 SDA/SS
D15                    D13         -- MFRC522 SCK
D16                    D11         -- MFRC522 MOSI
D14                    D12         -- MFRC522 MISO
GND                    GND         -- MFRC522 GND
D9                     D9          -- MFRC522 RST
3.3V (VCC)             3.3V        -- MFRC522 3.3V

5V                     5V          -- DFRobotDFPlayer VCC 
TX0                    D3          -- DFRobotDFPlayer RX  
RX1                    D2          -- DFRobotDFPlayer TX  
GND                    GND         -- DFRobotDFPlayer GND 
                                      DFRobotDFPlayer SPK_1 -- 25 Ohm speaker  
                                      DFRobotDFPlayer SPK_2 -- 25 Ohm speaker
*/

#define HAVE_NANO  1
//#define HAVE_MICRO 2



#include <SPI.h>
#include "MFRC522.h"
#include "MFRC522Hack.h"
#include <EEPROM.h>
#ifdef HAVE_NANO
  #include <SoftwareSerial.h>
#endif
#include "DFRobotDFPlayerMini.h"
#include "MPU6050.h"

//Set volume value. From 0 to 30
#define VOLUME 25    
#define SONG_COUNT  50

uint16_t tagToSong[SONG_COUNT] = {0};

MPU6050 mpu;

#define SS_PIN 10
#define RST_PIN 9
MFRC522 mfrc522(SS_PIN, RST_PIN);  // Create MFRC522 instance.
MFRC522Hack mfrc522Hack(&mfrc522);  // Create MFRC522Hack instance.

#define pinPowerOffSwitch 5 // for power-off circuit (optional)

#ifdef HAVE_NANO
  // Some arduino boards only have one hardware serial port, so a software serial port is needed instead.
  SoftwareSerial mySoftwareSerial(2, 3); // RX, TX
#endif

DFRobotDFPlayerMini myDFPlayer;

int count = 0;
unsigned long nextGyroTime = 0;
unsigned long activityTimeout = 0;
unsigned long saveTimeout = 0;
int idx = 0;
uint16_t tag = 0;
unsigned long stopTimeout = 0;
unsigned long powerOffTimeout = millis() + 20000; // initial power-off timeout (20 secs)
bool playing = false;
bool save = false;
Stream *playerSerial;

// Timers
unsigned long timer = 0;
float timeStep = 0.01;

// Pitch, Roll and Yaw values
float pitch = 0;
float roll = 0;
float yaw = 0;

// print TAG ID for each song
void printTags(){
  Serial.print(F("tagToSong: "));
  for (int i=0; i < SONG_COUNT; i++){    
    Serial.print(tagToSong[i]);    
    Serial.print(",");    
  }  
  Serial.println();
}

// clear TAG ID table
void clearEEPROM(){
  Serial.println(F("clearing EEPROM "));        
  unsigned long nextTime = 0;
  for (int addr=0; addr < 4096; addr++) {
    if (millis() > nextTime){
      nextTime = millis() + 2000;          
      Serial.print( (int) ((float(addr)) / 4096.0 * 100.0) ); 
      Serial.println( "%"); 
    }    
    EEPROM.write(addr, 0);      
  }
  for (int i=0; i < SONG_COUNT; i++) tagToSong[i]=0;
  printTags();
}

// save TAG ID table
void saveEEPROM(){
  Serial.println(F("saveEEPROM"));
  float floatValue;
  int addr = 0;
  unsigned short id = 0xAABF;
  EEPROM.put(addr, id);
  addr += sizeof(id);
  EEPROM.put(addr, tagToSong);
  addr += sizeof(tagToSong);   
}

// load TAG ID table
void loadEEPROM(){
  Serial.println(F("loadEEPROM"));
  int addr = 0;
  unsigned short id = 0;
  EEPROM.get(addr, id);  
  if (id != 0xAABF) {
    Serial.println(F("no valid EEPROM data found"));
    return;
  }
  Serial.println(F("valid EEPROM data found"));
  addr += sizeof(id);
  EEPROM.get(addr, tagToSong);
  addr += sizeof(tagToSong);
}

void setup() {
  pinMode(pinPowerOffSwitch, OUTPUT);
  digitalWrite(pinPowerOffSwitch, HIGH);
  
  Serial.begin(115200); // Initialize serial communications with the PC
  Serial.println(F("SETUP 27.10.2018"));
  
  SPI.begin();      // Init SPI bus
  
  #ifdef HAVE_NANO
    playerSerial = &mySoftwareSerial;    
    mySoftwareSerial.begin(9600);
  #else
    playerSerial = &Serial1;
    Serial1.begin(9600);
  #endif  
  
  randomSeed(analogRead(0));    

  Serial.println("mfrc522 init...");
  
  mfrc522.PCD_Init(); // Init MFRC522 card
  /*while (!mfrc522.PCD_PerformSelfTest()){
    Serial.println(F("Could not find MFRC522, check wiring!"));
    delay(500);
  }*/

  while(!mpu.begin(MPU6050_SCALE_2000DPS, MPU6050_RANGE_2G))
  {
    Serial.println(F("Could not find a valid MPU6050 sensor, check wiring!"));
    delay(5000);
  }
  /*mpu.calibrateGyro();
  mpu.setThreshold(3);
  
  mpu.setAccelPowerOnDelay(MPU6050_DELAY_3MS);
  mpu.setIntFreeFallEnabled(false);  
  mpu.setIntZeroMotionEnabled(false);
  mpu.setIntMotionEnabled(false); 
  mpu.setDHPFMode(MPU6050_DHPF_5HZ);
  mpu.setMotionDetectionThreshold(10);
  mpu.setMotionDetectionDuration(10);
  mpu.setZeroMotionDetectionThreshold(4);
  mpu.setZeroMotionDetectionDuration(2);    */
  
  while (!myDFPlayer.begin(*playerSerial)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin DFRobotDFPlayerMini:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    delay(5000);
  }
  Serial.println(F("DFPlayer Mini online."));  
  

  myDFPlayer.setTimeOut(500); //Set serial communictaion time out 500ms
  myDFPlayer.volume(VOLUME);  //Set volume value. From 0 to 30
  myDFPlayer.EQ(DFPLAYER_EQ_NORMAL);
  myDFPlayer.outputDevice(DFPLAYER_DEVICE_SD);
  
  while (count == 0){
    Serial.print(F("files:"));    
    count = SONG_COUNT; //myDFPlayer.readFileCounts();
    Serial.println(count);  
    delay(1000);    
  }

  loadEEPROM();
  printTags();
       
  //myDFPlayer.play(idx);  //Play the first mp3  
 
  Serial.println(F("Scan PICC to see UID and type..."));
}

// pause playing song
void pause(){
  Serial.println(F("PAUSE"));
  myDFPlayer.pause();  
  playing = false;  
}

// play song with idx
void play(int idx){
  Serial.print(F("PLAYING "));
  Serial.println(idx+1);
  myDFPlayer.play(idx+1);        
  playing = true;  
  powerOffTimeout = millis() + 600000; // reset power-off timeout (600 secs)
}

// play song with next idx
void playNext(){
  idx++;
  if (idx >= SONG_COUNT) idx = 0;
  play(idx);
}

// play song with previous idx
void playPrevious(){
  idx--;
  if (idx < 0) idx = SONG_COUNT-1;      
  play(idx);
}

void loop() {    
  // Read normalized values 
  Vector normAccel = mpu.readNormalizeAccel();

  // Calculate Pitch & Roll
  float vPitch = -(atan2(normAccel.XAxis, sqrt(normAccel.YAxis*normAccel.YAxis + normAccel.ZAxis*normAccel.ZAxis))*180.0)/M_PI;
  float vRoll = (atan2(normAccel.YAxis, normAccel.ZAxis)*180.0)/M_PI;
  pitch = pitch * 0.9 + vPitch * 0.1;
  roll = roll   * 0.9 + vRoll  * 0.1;
 
         
  if (millis() > activityTimeout){    
    if (pitch > 30) {            
      // TAG programming mode: choose next song for TAG
      activityTimeout = millis() + 2000;
      saveTimeout = millis() + 2000;
      Serial.println(F("--->"));                  
      if (playing){
        playNext();
        save = true;
      }
    }
    else if (pitch < -30) {
      // TAG programming mode: choose previous song for TAG
      activityTimeout = millis() + 2000;      
      saveTimeout = millis() + 5000;
      Serial.println(F("<---"));            
      if (playing){
        playPrevious();                
        save = true;
      }      
    } else {
      if (save){           
        if (playing){
          // TAG programming mode finished: save TAG in TAG ID table
          for (int i=0; i < SONG_COUNT; i++) if (tagToSong[i] == tag) tagToSong[i]=0;
          tagToSong[idx]=tag;      
          saveEEPROM();
          printTags();
        }
        save = false;
      }
    }
  } 
  
  
  if (Serial.available()){
    // serial console test functions
    char ch = Serial.read();
    if (ch == 'c') {
      clearEEPROM();
    }
    if (ch == '+') {
      playNext();      
    }
    if (ch == '-') {
      playPrevious();      
    }
    if (ch == 'r') {
      // Set new UID to card            
      byte newUid[] = {0x00, 0x00, 0x00, 0x00};
      newUid[0] = random(0, 256);                
      newUid[1] = random(0, 256);                
      newUid[2] = random(0, 256);                
      newUid[3] = random(0, 256);                      
      if ( mfrc522Hack.MIFARE_SetUid(newUid, (byte)4, true) ) {
        Serial.println(F("Wrote new UID to card."));
        mfrc522.PICC_HaltA();
      }
    }   
  }

  if (millis() > stopTimeout){
    if (playing) {
      pause();       // pause playing
    }    
  }
      
  // Look for new cards
  if ( mfrc522.PICC_IsNewCardPresent()) {  
    // Select one of the cards
    if ( mfrc522.PICC_ReadCardSerial()) {
      // make unique key based on serial number 
      uint16_t v = 0;
      for (byte i = 0; i < mfrc522.uid.size; i++) {
        v += (i+1) * mfrc522.uid.uidByte[i];    
      } 
      /* // add some card data to key 
         byte dataBlock[]    = { 0 };            
         if (mfrc522.MIFARE_Read(4, dataBlock, 4) == MFRC522::STATUS_OK) {
           for (byte i = 0; i < 4; i++) v += dataBlock[i];        
         } */     
      // check if there is a new unique key
      if (v != tag){
        tag = v;
        Serial.print(F("new TAG: "));
        Serial.println(tag);        
        //idx = 0;
        for (int i=0; i < SONG_COUNT; i++){
          if (tagToSong[i] == tag){ // found TAG ID in TAG ID table
            Serial.println(F("found in tagToSong"));
            idx = i;
            break;
          }
        }
        //idx = random(1, count+1);                
        play(idx); // play new song                
      } else { // same card again
        if (millis() > stopTimeout) { // unpause playing
          Serial.println(F("START"));
          myDFPlayer.start();        
          playing = true;
        }
      }  
      stopTimeout = millis() + 1000;    
    }      
  }  

  if (millis() > powerOffTimeout){
    // no new song after timeout => switch off player
    powerOffTimeout = millis() + 20000;
    Serial.println(F("powering off..."));        
    digitalWrite(pinPowerOffSwitch, LOW);
  }
}



